@@ -65,7 +65,7 @@ extern char *gnt_key_cright;
 #define GNT_KEY_BACKSPACE SAFE(key_backspace)
 #define GNT_KEY_DEL    SAFE(key_dc)
 #define GNT_KEY_INS    SAFE(key_ic)
-#define GNT_KEY_BACK_TAB SAFE(back_tab)
+#define GNT_KEY_BACK_TAB (back_tab ? back_tab : SAFE(key_btab))
 
 #define GNT_KEY_CTRL_A     "\001"
 #define GNT_KEY_CTRL_B     "\002"
@@ -158,7 +158,7 @@ void gnt_keys_del_combination(const char *key);
  */
 int gnt_keys_find_combination(const char *key);
 
-/* A lot of commonly used variable names are defined in <term.h>. 
+/* A lot of commonly used variable names are defined in <term.h>.
  * #undef them to make life easier for everyone. */
 
 #undef columns
