@@ -366,3 +366,44 @@ purple_smarshal_VOID__STRING_STRING_OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
+/* VOID:POINTER,POINTER,OBJECT,OBJECT (./marshallers.list:9) */
+void
+purple_smarshal_VOID__POINTER_POINTER_OBJECT_OBJECT (GClosure     *closure,
+                                                     GValue       *return_value G_GNUC_UNUSED,
+                                                     guint         n_param_values,
+                                                     const GValue *param_values,
+                                                     gpointer      invocation_hint G_GNUC_UNUSED,
+                                                     gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__POINTER_POINTER_OBJECT_OBJECT) (gpointer     data1,
+                                                                    gpointer     arg_1,
+                                                                    gpointer     arg_2,
+                                                                    gpointer     arg_3,
+                                                                    gpointer     arg_4,
+                                                                    gpointer     data2);
+  register GMarshalFunc_VOID__POINTER_POINTER_OBJECT_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 5);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__POINTER_POINTER_OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_pointer (param_values + 1),
+            g_marshal_value_peek_pointer (param_values + 2),
+            g_marshal_value_peek_object (param_values + 3),
+            g_marshal_value_peek_object (param_values + 4),
+            data2);
+}
+
