@@ -184,8 +184,8 @@ digest_md5_handle_challenge(JabberStream *js, xmlnode *packet,
 	dec_in = (char *)purple_base64_decode(enc_in, NULL);
 	purple_debug_misc("jabber", "decoded challenge (%"
 			G_GSIZE_FORMAT "): %s\n",
-			dec_in != NULL ? strlen(dec_in) : 0,
-			dec_in != NULL  ? dec_in : "(null)");
+			strlen(dec_in),
+			dec_in);
 
 	parts = jabber_auth_digest_md5_parse(dec_in);
 
